import React from 'react';
import { createProvider } from './create-provider';
import { tryCreateDevToolsLogger } from './devtools';
export function createStore(initialState, actions, extraArgument) {
    let getState;
    let setState;
    const Context = React.createContext(initialState);
    const Provider = createProvider((providerGetState, providerSetState) => {
        getState = providerGetState;
        setState = (state) => {
            if (typeof state === 'function') {
                return state(providerGetState, providerSetState, extraArgument);
            }
            else {
                devToolsLogger && devToolsLogger('state update', state);
                providerSetState(state);
            }
        };
    }, Context.Provider, initialState);
    const devToolsLogger = tryCreateDevToolsLogger();
    const wrappedActions = {};
    Object.keys(actions)
        .forEach(key => {
        const originalFn = actions[key];
        const wrappedFn = (...args) => {
            const result = originalFn(...args);
            if (!getState || !setState) {
                // tslint:disable-next-line:no-console
                console.error('<Provider /> is not initialized');
                return;
            }
            if (typeof result === 'function') {
                devToolsLogger && devToolsLogger(`thunk — ${key}`, getState());
                return result(getState, setState, extraArgument);
            }
            else {
                // Never-ever goes here
                devToolsLogger && devToolsLogger(key, result);
                setState(result);
                return;
            }
        };
        wrappedActions[key] = wrappedFn;
    });
    return {
        Context,
        Provider,
        Consumer: Context.Consumer,
        actions: wrappedActions,
        getState: () => getState ? getState() : initialState,
    };
}
